import React from 'react';
export type WithQueryProps<QueryParams> = {
    location: {
        search?: string;
    };
    queryParams?: QueryParams;
};
/**
 * A HoC which augments the props with the queryParams from the
 * react-router-dom's `Location.search` string as an actual object.
 *
 * Requires defaultQueryParameters to fill in the blanks when the
 * query parameters are not in the url. This way you are guaranteed
 * to always have values for your query parameters.
 *
 * It also uses the defaultQueryParameters to convert the query
 * parameters (which all come out as strings) to the type of
 * the defaultQueryParameter.
 *
 * @example
 * withQueryParams(App, { page: 1, search: '', showAll: true })
 *
 * This will create an App component which has a prop queryParams which
 * is an object which contains page a number, search a string, and
 * showAll a boolean.
 *
 * @example
 * withQueryParams(App, { show: ['medium'] })
 *
 * Can also support arrays and convert to actual types as well.
 *
 * @param {React.ComponentType<P>} Component The react component which you want to encapsulate with the withQueryParams HoC
 * @param {Object} defaultQueryParams The default query parameters which will be used to fill in any blanks in the query params
 * @returns Component wrapped with HoC which provides the query params as a prop.
 */
export declare const withQueryParams: <P extends Record<string, unknown>, QueryParams extends Record<string, unknown>>(Component: React.ComponentType<P>, defaultQueryParams: QueryParams) => React.ComponentClass<P & WithQueryProps<QueryParams>>;
